//
// C++ Implementation:
//
// Description:
//
//
// Author: Jean-Michel PETIT <k9copy@free.fr>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "k9asttitles.h"
#include "k9config.h"
#include "k9dvd.h"
#include "k9dvdtitle.h"
#include <QObject>
#include "kviewmpeg2.h"
#include <QLayout>
#include <QCheckBox>
_k9ItemWidget::_k9ItemWidget(k9DVDTitle *_title,QTreeWidget *_treeWidget,int _type):QObject(0),QTreeWidgetItem(_treeWidget,_type) {
    switch (_type) {
    case k9astTitles::TITLE :
        title=_title;
        setFlags(Qt::ItemIsEnabled | Qt::ItemIsUserCheckable | Qt::ItemIsSelectable);
        setCheckState(0,Qt::Unchecked);
        connect (title,SIGNAL(selectionChanged()),this,SLOT(titleChanged()));
        break;
    default:
        break;
    }

}

void _k9ItemWidget::titleChanged() {
    if (title->isSelected())  {
        setCheckState(0,Qt::Checked);
        astTitle->titleSelected(title);
    } else {
        setCheckState(0,Qt::Unchecked);
        astTitle->titleUnSelected(title);
    }
}

bool _k9ItemWidget::operator< ( const QTreeWidgetItem & other ) const  {
    switch (treeWidget()->sortColumn()) {
	case 1:
            return title->getnumTitle() <((_k9ItemWidget*)&other)->title->getnumTitle();
 	case 2:
	    return title->gettotalsize_mb() <((_k9ItemWidget*)&other)->title->gettotalsize_mb();
 	default:
	    return 1;
     }
}

k9astTitles::k9astTitles(QWidget* parent,k9DVD *_dvd)
        : QWidget(parent) {
    Ui_astTitles.setupUi(this);
    m_dvd=_dvd;
    m_src=Ui_astTitles.twSource;
    m_preview=NULL;
    loadPlayer();
    QTreeWidgetItem *header=m_src->headerItem();
    header->setText(0,"");
    QWidget *w=new QWidget(m_src);
    QCheckBox *ck=new QCheckBox(w);
    ck->setChecked(true);
    connect(ck,SIGNAL(toggled( bool )),this,SLOT(selectAll(bool)));
    
    QGridLayout *layout=new QGridLayout(w);
    w->setContentsMargins(0,0,0,0);
    layout->addWidget(ck,0,0);
    m_src->setItemWidget(header,0,w);
    m_src->setSortingEnabled(true);
}


void k9astTitles::loadPlayer() {
    k9Config config;
    m_useXine=config.getUseXine();
    m_usePhonon=config.getUsePhonon();
    m_useMplayer=config.getUseMplayer();

    if (m_preview) {
  	delete m_preview;
        delete m_layout;
    }

    if (m_useXine) {
  	m_preview=new k9xine(Ui_astTitles.wPreview);
    }
    else if (m_usePhonon) {
	m_preview=new k9Phonon(Ui_astTitles.wPreview);
    }else if (m_useMplayer)
	m_preview=new K9Mplayer(Ui_astTitles.wPreview);
    else
        m_preview=new kViewMPEG2(Ui_astTitles.wPreview);

   connect(this,SIGNAL(sigshowPreview( k9DVD*, k9DVDTitle*,int )),m_preview,SLOT(open( k9DVD*, k9DVDTitle*,int )));
   connect(this,SIGNAL(sigstopPreview()),m_preview,SLOT(bStopClick()));

    m_layout=new QGridLayout(Ui_astTitles.wPreview);
    m_layout->addWidget(m_preview,0,0);
    m_preview->show();

}

void k9astTitles::itemActivated(QTreeWidgetItem* _item,int) {
   _k9ItemWidget *item=(_k9ItemWidget*)_item;
   if (item) {
   //   emit sigstopPreview();;
      emit sigshowPreview(m_dvd,item->title,1);
   }
  
}

k9astTitles::~k9astTitles() {
   emit sigstopPreview();
}

void k9astTitles::unloadPreview() {
  delete m_preview;
}
/*$SPECIALIZATION$*/

void k9astTitles::stopPreview() {
   emit sigstopPreview();
}

void k9astTitles::titleSelected(k9DVDTitle *_title) {
    if (!m_updating)
        emit addTitle(_title);
}

void k9astTitles::titleUnSelected(k9DVDTitle *_title) {
    if (!m_updating)
        emit removeTitle(_title);
}

void k9astTitles::selectAll(bool _state) {
   bool oldState;
   for(int i=0; i < m_src->topLevelItemCount();i++) {
      _k9ItemWidget *item=(_k9ItemWidget*) m_src->topLevelItem(i);
      oldState=item->title->isSelected();
      if (_state) {
	    m_updating=true;
	   // all streams are checked by default
            for (int j=0;j < item->title->getaudioStreamCount();j++) {
                item->title->getaudioStream(j)->setselected(true);
		if (j==0)
		   item->title->setDefAudio(item->title->getaudioStream(j));
	    }
            for (int j=0;j < item->title->getsubPictureCount();j++) {
                item->title->getsubtitle(j)->setselected(true);
		if (j==0)
		   item->title->setDefSubtitle(item->title->getsubtitle(j));
 	    }
	    item->title->setvideoSelected(true);
  	    m_updating=false;
	    if (_state!=oldState)
		emit addTitle(item->title);
      }else {
 	    m_updating=true;
            item->title->setSelected(false);
	    item->title->setDefSubtitle(NULL);
	    item->title->setDefAudio(NULL);
	    m_updating=false;
	    if(_state!=oldState)
		emit removeTitle(item->title);
      }
   }
}

void k9astTitles::fill() {
    m_src->clear();


    for (int i=0;i < m_dvd->gettitleCount();i++) {
        k9DVDTitle *title=m_dvd->gettitle(i);
        if (title->getIndexed()) {
            m_updating=true;

            _k9ItemWidget *item=new _k9ItemWidget(title,m_src,k9astTitles::TITLE );
            item->astTitle=this;
            item->setText(0,"");
            item->setText(1,QString("%1").arg(title->getname()));
            item->setText(2,QString("%1 %2").arg(title->gettotalsize_mb(),0,'f',2).arg(i18n("MB")));

            // all streams are checked by default
            title->setSelected(true);
            m_updating=false;
            emit addTitle(title);
        }
    }
    m_src->sortItems(1,Qt::AscendingOrder);
}


void k9astTitles::resizeEvent ( QResizeEvent * event ) {
     m_src->setColumnWidth(1,m_src->viewport()->width());
    m_src->resizeColumnToContents(0);
    m_src->resizeColumnToContents(2);

    m_src->setColumnWidth(1,m_src->viewport()->width()-m_src->columnWidth(0)-m_src->columnWidth(2));


}

void k9astTitles::updateTitles() {
    for (int j=0;j< m_src->topLevelItemCount();j++) {
        _k9ItemWidget *item=(_k9ItemWidget*)m_src->topLevelItem(j);
        if (item->checkState(0)==Qt::Unchecked) {
            item->title->setSelected(false);

        } else if (!item->title->isSelected()) {
            m_updating=true;
            item->title->setSelected(true);
            m_updating=false;
            emit addTitle(item->title);
        }
    }
//reselectionner les titres qui ont été recochés
}

bool k9astTitles::isOk() {
    bool ok=false;
    for (int i=0;i < m_dvd->gettitleCount();i++) {
        k9DVDTitle *title=m_dvd->gettitle(i);
        ok= (title->getIndexed() && title->isSelected())?true:ok; 
	  
   }
   return ok;

}

